//-------------------------------------
// Based on "C++ Win32 Multi-Threaded Rubik Cube Sample"
// Copyright 1987-2002 Amzi! inc.
//-------------------------------------
// Additional programming 2003 Sergey Vlasov
//-------------------------------------

#include "Rubik.h"
#include <windows.h>

#pragma comment(lib, "amzi.lib")

/////////////////////////////////////////////////////////////////
// Extended Prolog predicates

TF EXPFUNC p_bugmsg(VOIDptr pr){
	return TRUE;
}

TF EXPFUNC p_cube_print(VOIDptr pr){
	return TRUE;
}

TF EXPFUNC p_wrfield(VOIDptr pr){
	return TRUE;
}

TF EXPFUNC p_disp_hist(VOIDptr pr){
	return ((Rubik*)pr)->disp_hist();
}

TF EXPFUNC p_my_cube_init(VOIDptr pr){
	return ((Rubik*)pr)->cube_init();
}

int g_tilemap[55] = {0,
	40, 31, 22, 4, 13, 49, 38, 33, 26,
	44, 35, 47, 36, 17, 24, 42, 15, 45,
	8, 27, 20, 2, 29, 53, 6, 11, 18,
	0, 9, 51, 30, 23, 32, 50, 14, 21,
	12, 48, 37, 25, 43, 46, 7, 19, 1,
	52, 41, 34, 39, 16, 5, 28, 3, 10};


Rubik::Rubik() : solved_(false), error_description_("Not tried to solve."){
} // ensure: !is_solved()

void Rubik::solve(int cube_position_to_solve_[54]){	// require: !is_solved()
	std::copy(cube_position_to_solve_, cube_position_to_solve_ + 54, cube_position_to_solve);

	try	{
		Init("rubik");
		AddPred("bugmsg", 1, &::p_bugmsg, this);
		AddPred("cube_print", 1, &::p_cube_print, this);
		AddPred("wrfield", 2, &::p_wrfield, this);
		AddPred("disp_hist", 1, &::p_disp_hist, this);
		AddPred("external_cube_init", 0, &::p_my_cube_init, this);
		Load("rubik");

		TERM t;
		if (ExecStr(&t, "wininit") == FALSE)
			throw std::exception("wininit error");
		if (ExecStr(&t, "solve(external)") == FALSE)
			throw std::exception("can't solve");

		Close();
		solved_ = true;
	}
	catch (CLSException &e)	{
		char msg[512];
		e.GetMsg(msg, 511);
		error_description_ = msg;
	}
	catch (std::exception &e)	{
		error_description_ = e.what();
	}
}

namespace{
	std::string cube_as_frubld_string(int engine_cube[54]){
		std::string result;
		char c;
		for(int i = 0; i < 54; ++i){
			result += "'";
			switch(engine_cube[i])
			{
			case 0: c='F'; break;
			case 1: c='R'; break;
			case 2: c='U'; break;
			case 3: c='B'; break;
			case 4: c='L'; break;
			case 5: c='D'; break;
			}
			result += c;
			result += "'";
			if (i != 53)
				result += ",";
		}
		return result;
	}

	void re_tile_to_engine(int normal[54], int engine[54]){
		for(int i = 0; i < 54; ++i)
			engine[i] = normal[g_tilemap[i + 1]];
	}
}

TF Rubik::cube_init()
{
	TERM t;
    ExecStr(&t, "retractall(state(_))");

	int engine_cube[54];
	re_tile_to_engine(cube_position_to_solve, engine_cube);
	std::string cube = "cube(" + cube_as_frubld_string(engine_cube) + ")";
	char data[500];
	lstrcpy(data, ("assert(state(" + cube + "))").c_str());
	ExecStr(&t, data);

	lstrcpy(data, ("new_colors(" + cube + ")").c_str());
	ExecStr(&t, data);

	return TRUE;
}

TF Rubik::disp_hist(){
	char hist[80];
	GetParm(1, cSTR, hist);
	solution_.push_back(hist);
	return TRUE;
}

bool Rubik::is_solved() const{
	return solved_;
}

std::vector<std::string> Rubik::solution() const{	// require: is_solved()
	return solution_;
}

std::string Rubik::error_description() const{	// require: !is_solved()
	return error_description_;
}