indexing
	notes: "Value type"
	
class
	ANGLE3D
	
inherit
	MATH_MISC
		redefine 
			copy, is_equal 
		end
	
create 
	make
	
feature hor_angle, vert_angle: ANGLE
	
feature make(new_hor_angle, new_vert_angle: like hor_angle) is
		require
			new_hor_angle /= Void
			new_vert_angle /= Void
		do
			hor_angle := clone(new_hor_angle)
			vert_angle := clone(new_vert_angle)
		end
	
feature to_point(distance: DOUBLE): VECTOR_3 is
		local 
			xy: DOUBLE
			x, y, z: REAL
		do
			z := feature {MATH}.sin(deg_to_rad(vert_angle)) * distance
			xy := feature {MATH}.cos(deg_to_rad(vert_angle)) * distance
			x := feature {MATH}.cos(deg_to_rad(hor_angle)) * xy
			y := feature {MATH}.sin(deg_to_rad(hor_angle)) * xy
			Result.make(x, y, z)
		end
	
feature infix "+" (other: like Current): like Current is
		require
			other /= Void
		do
			Result := clone(Current)
			Result.add(other)
		ensure
			Result /= Void
		end
	
feature infix "-" (other: like Current): like Current is
		require
			other /= Void
		do
			Result := clone(Current)
			Result.sub(other)
		ensure
			Result /= Void
		end
	
feature add(other: like Current) is
		require
			other /= Void
		do
			hor_angle.add(other.hor_angle) 
			vert_angle.add(other.vert_angle) 
		end
	
feature sub(other: like Current) is
		require
			other /= Void
		do
			hor_angle.sub(other.hor_angle) 
			vert_angle.sub(other.vert_angle) 
		end
	
feature copy(other: like Current) is
		do
			hor_angle := clone(other.hor_angle)
			vert_angle := clone(other.vert_angle)
		end	
	
feature is_equal(other: like Current): BOOLEAN is
		do
			Result := hor_angle.is_equal(other.hor_angle) and vert_angle.is_equal(other.vert_angle)
		end		
	
invariant
	hor_angle /= Void
	vert_angle /= Void
	
end 
