
class
	BRICK_PAINTER
	
inherit
	MISC	
	MATH_MISC
	
create
	make
	
	
feature {NONE} make(new_render_device: DEVICE) is
		require
			new_render_device /= Void
		do
			render_device := new_render_device
			brick_mesh := feature {MESH}.from_stream(load_resource_as_stream("brick.x", "cube resources"), 
													 feature {MESH_FLAGS}.system_memory, 
													 render_device)
		end
	
	
feature draw(brick: BRICK) is
		require
			brick /= Void
		local
			t: MATRIX
		do
			t := feature {MATRIX}.translation(brick.position)
			render_device.Transform.set_world(t * rotation_matrix(brick.angle))
			
			draw_subset(drawing_color(brick.colors.select_color(Direction.Front)), 0)
			draw_subset(drawing_color(brick.colors.select_color(Direction.Left)), 2)
			draw_subset(drawing_color(brick.colors.select_color(Direction.Right)), 5)
			draw_subset(drawing_color(brick.colors.select_color(Direction.Back)), 3)
			draw_subset(drawing_color(brick.colors.select_color(Direction.Top)), 1)
			draw_subset(drawing_color(brick.colors.select_color(Direction.Bottom)), 4)									
			draw_subset(feature {DRAWING_COLOR}.from_argb(60, 60, 60), 6)												
		end
	
feature {NONE} draw_subset(color: DRAWING_COLOR subset: INTEGER) is
		local
			m: MATERIAL
		do
			m.set_emissive(color)
			render_device.set_material(m)
			brick_mesh.draw_subset(subset)
		end		
	
feature {NONE} rotation_matrix(angle: ROTATION_ANGLE): MATRIX is
		require
			angle /= Void
		local 
			rotate_x, rotate_y, rotate_z: MATRIX
		do
			rotate_x.rotate_x(deg_to_rad(angle.around_x))
			rotate_y.rotate_y(deg_to_rad(angle.around_y))
			rotate_z.rotate_z(deg_to_rad(angle.around_z))
				-- Result := rotate_x * rotate_y * rotate_z
			Result := rotate_x * rotate_y
			Result := Result * rotate_z
		end	
	
feature {NONE} drawing_color(color: INTEGER): DRAWING_COLOR is 
		do
			inspect 
				color
			when feature {COLOR}.Red then
				Result := feature {DRAWING_COLOR}.red
			when feature {COLOR}.Yellow then
				Result := feature {DRAWING_COLOR}.yellow
			when feature {COLOR}.Green then
				Result := feature {DRAWING_COLOR}.green
			when feature {COLOR}.Blue then
				Result := feature {DRAWING_COLOR}.blue
			when feature {COLOR}.White then
				Result := feature {DRAWING_COLOR}.white
			when feature {COLOR}.Pink then
				Result := feature {DRAWING_COLOR}.pink
			when feature {COLOR}.Black then
				Result := feature {DRAWING_COLOR}.black
			end
		end	
	
feature	{NONE} Direction: DIRECTION is
		once
			create Result
		end           
	
feature {NONE}
	brick_mesh: MESH
	render_device: DEVICE
	
invariant
	brick_mesh /= Void
	render_device /= Void
	
end 
