
class
	CUBE
	
inherit
	RENDERABLE 
	
create 
	make
	
feature ideal_cube: IDEAL_CUBE
	
feature {NONE} make(new_offset_between_bricks_centers: like offset_between_bricks_centers) is
		require
			new_offset_between_bricks_centers >= 1.0
		do
			offset_between_bricks_centers := new_offset_between_bricks_centers
			create mutex.make
			create ideal_cube
		end
	
feature rotate_plane(new_plane: like plane; new_angle: like angle) is
		require
			new_plane /= Void
			new_angle /= Void
		do
			dummy := mutex.wait_one
			rotate_mode := True
			angle := new_angle
			plane := new_plane
			mutex.release_mutex
		ensure
			rotate_mode = True	
		end
	
feature rotate_ideal_cube(move: PLAYER_MOVE) is
		require
			move /= Void
		do
			dummy := mutex.wait_one
			rotate_mode := False
			ideal_cube.play_move(move)
			mutex.release_mutex
		ensure
			rotate_mode = False
		end
	
feature set_ideal_cube(cube: like ideal_cube) is
		require
			cube /= Void
		do
			ideal_cube := cube
		end
	
feature render(brick_painter: BRICK_PAINTER) is
		do
			make_bricks.do_all(agent brick_painter.draw)
		end
	
feature {NONE} make_bricks: ARRAYED_LIST[BRICK] is
		local
			delta_angle: ROTATION_ANGLE
		do
			create Result.make(feature {IDEAL_CUBE}.Total_bricks)
			dummy := mutex.wait_one
			if rotate_mode then 
				delta_angle := rotation_angle_from_angle(plane.orientation, angle)
				extend_bricks(Result, ideal_cube.select_plane(plane), delta_angle)
				extend_bricks(Result, ideal_cube.select_all_except_plane(plane), Zero_angle)
			else
				extend_bricks(Result, ideal_cube.bricks, Zero_angle)
			end
			mutex.release_mutex
		ensure
			Result /= Void 
			Result.count = feature {IDEAL_CUBE}.Total_bricks 
			not Result.has(Void)	
		end
	
	
feature {NONE} extend_bricks(bricks_to_extend: ARRAYED_LIST[BRICK] bricks_to_add: ARRAYED_LIST[IDEAL_BRICK] rangle: ROTATION_ANGLE) is
		require
			bricks_to_extend /= Void and then not bricks_to_extend.has(Void)
			bricks_to_add /= Void and then not bricks_to_add.has(Void)
			rangle /= Void
		do
			--bricks_to_add.do_all(agent bricks_to_extend.extend(agent brick_point_to_brick(?, rangle)))
			from bricks_to_add.start until bricks_to_add.off loop
				bricks_to_extend.extend(ideal_brick_to_brick(bricks_to_add.item, rangle))
				bricks_to_add.forth
			end
		ensure
			bricks_to_extend.count = old bricks_to_extend.count + bricks_to_add.count
			not bricks_to_extend.has(Void)
		end
	
feature {NONE} ideal_brick_to_brick(ideal_brick: IDEAL_BRICK rotation_angle: ROTATION_ANGLE): BRICK is
		require
			ideal_brick /= Void
			rotation_angle /= Void
		local
			x, y, z: REAL
		do
			x := ideal_brick.position.x.value * offset_between_bricks_centers
			y := ideal_brick.position.y.value * offset_between_bricks_centers
			z := ideal_brick.position.z.value * offset_between_bricks_centers
			create Result.make(ideal_brick.colors, create {VECTOR_3}.make(x, y, z), rotation_angle)
		ensure
			Result /= Void
		end
	
feature {NONE} rotation_angle_from_angle(orientation: INTEGER simple_angle: ANGLE): ROTATION_ANGLE is
		require
			simple_angle /= Void
		do
			inspect 
				orientation
			when feature {ORIENTATION}.Vertical_perpendicular_to_x then 
				create Result.make(simple_angle, Zero, Zero)
			when feature {ORIENTATION}.Vertical_perpendicular_to_y then 
				create Result.make(Zero, simple_angle, Zero)
			when feature {ORIENTATION}.Horizontal then
				create Result.make(Zero, Zero, simple_angle)
			end	
		ensure
			Result /= Void
		end	
	
feature {NONE} Zero_angle: ROTATION_ANGLE is
		once
			create Result.make(Zero, Zero, Zero)
		end	
	
feature {NONE} Zero: ANGLE is
		once
			create Result.make(0)
		end	
	
	
feature {NONE} 
	plane: CUBE_PLANE_PARAMETERS
	rotate_mode: BOOLEAN
	angle: ANGLE
	offset_between_bricks_centers: DOUBLE
	mutex: SYSTEM_MUTEX
	dummy: BOOLEAN
	
invariant
	ideal_cube /= Void
	mutex /= Void
	rotate_mode implies angle /= Void and plane /= Void
	offset_between_bricks_centers >= 1.0
	
end 
