
class
	CUBE_MODEL
	
inherit
	MODEL
		rename
			make as base_make, scene as cube
		export {TEST_CUBE_MODEL}
			get_spectator
		redefine
			cube, run, stop
		end
	EXCEPTIONS
	MATH_MISC
	MISC
	CUBE_SOLVER
	
create
	make
	
feature {NONE} make(new_spectator: like spectator; new_cube: like cube; new_shuffle_count: like shuffle_count) is
		require
			new_spectator /= Void and then valid_spectator(new_spectator)
			new_cube /= Void
			new_shuffle_count >= 0
		do
			create stopping
			create celebrating
			demo_stopped := True
			cube := new_cube
			shuffle_count := new_shuffle_count
			base_make(new_spectator, create {VECTOR_3}.make)
			initialize_key_handlers			
		end
	
feature
	Minimal_spectator_distance: DOUBLE is 4.0
	Maximal_spectator_distance: DOUBLE is 80.0
	Maximal_spectator_vertical_angle: DOUBLE is 60.0
	
feature valid_spectator(s: SPECTATOR): BOOLEAN is
		require
			s /= Void
		do
			Result := valid_spectator_distance(s.distance) and valid_spectator_vert_angle(s.angle.vert_angle)
		end
	
feature valid_spectator_distance(distance: DOUBLE): BOOLEAN is
		do
			Result := distance >= Minimal_spectator_distance and distance <= Maximal_spectator_distance 
		end
	
feature valid_spectator_vert_angle(angle: ANGLE): BOOLEAN is		
		require
			angle /= Void
		do
			Result := angle.value >= 360.0 - Maximal_spectator_vertical_angle or angle.value <= Maximal_spectator_vertical_angle
		end
	
feature run is
		local
			worker_thread: SYSTEM_THREAD
		do
			create worker_thread.make(create {THREAD_START}.make(Current, $process_cube))
			worker_thread.set_priority(feature {THREAD_PRIORITY}.below_normal)
			worker_thread.start
		end
	
feature stop is
		do
			stopping := True
			from until demo_stopped.is_true loop
				feature {WINFORMS_APPLICATION}.do_events
			end
		ensure then
			demo_stopped.is_true
		end
	
feature {TEST_CUBE_MODEL} -- Key press handling
	
	initialize_key_handlers is
		local
			key_handler: KEY_HANDLER
		do
			create key_handler.make(feature {WINFORMS_KEYS}.right, agent move_spectator_right, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.left, agent move_spectator_left, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.up, agent move_spectator_up, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.down, agent move_spectator_down, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.add, agent move_spectator_closer, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.subtract, agent move_spectator_farther, agent not_celebrating)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.F2, agent save_cube, agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.F3, agent load_cube, agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_1, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_x, -1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_2, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_x, 0), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_3, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_x, 1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_4, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_y, -1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_5, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_y, 0), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_6, 
									agent user_rotate(feature {ORIENTATION}.Vertical_perpendicular_to_y, 1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_7, 
									agent user_rotate(feature {ORIENTATION}.Horizontal, -1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_8, 
									agent user_rotate(feature {ORIENTATION}.Horizontal, 0), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
			
			create key_handler.make(feature {WINFORMS_KEYS}.num_pad_9, 
									agent user_rotate(feature {ORIENTATION}.Horizontal, 1), 
									agent is_demo_stopped)
			add_key_handler(key_handler)
		end
	
	not_celebrating: BOOLEAN is 	
		do
			Result := not celebrating
		end
	
	is_demo_stopped: BOOLEAN is 	
		do
			Result := demo_stopped.is_true
		end
	
	update_spectator_angle(new_angle: ANGLE3D) is
		require
			new_angle /= Void and then valid_spectator_vert_angle(new_angle.vert_angle)	
		do
			set_spectator(create {SPECTATOR}.make(new_angle, get_spectator.distance))
		ensure
			-- observer notified
		end
	
	update_spectator_distance(new_distance: DOUBLE) is
		require
			valid_spectator_distance(new_distance)
		do
			set_spectator(create {SPECTATOR}.make(get_spectator.angle, new_distance))
		ensure
			-- observer notified
		end
	
	
	move_spectator_right is
		do
			update_spectator_angle(get_spectator.angle - Spectator_hor_angle_change_delta)
		end
	
	move_spectator_left is
		do
			update_spectator_angle(get_spectator.angle + Spectator_hor_angle_change_delta)
		end
	
	move_spectator_up is
		local 
			new_angle: ANGLE3D
		do
			new_angle := get_spectator.angle + Spectator_vert_angle_change_delta
			if valid_spectator_vert_angle(new_angle.vert_angle) then
				update_spectator_angle(new_angle)
			end
		end
	
	move_spectator_down is
		local 
			new_angle: ANGLE3D
		do
			new_angle := get_spectator.angle - Spectator_vert_angle_change_delta
			if valid_spectator_vert_angle(new_angle.vert_angle) then
				update_spectator_angle(new_angle)
			end
		end
	
	move_spectator_farther is
		local
			new_distance: DOUBLE
		do
			new_distance := get_spectator.distance * Spectator_distance_change_rate
			if valid_spectator_distance(new_distance) then
				update_spectator_distance(new_distance)
			end
		end
	
	move_spectator_closer is
		local
			new_distance: DOUBLE
		do
			new_distance := get_spectator.distance / Spectator_distance_change_rate
			if valid_spectator_distance(new_distance) then
				update_spectator_distance(new_distance)
			end
		end
	
	user_rotate(plane_orientation: INTEGER plane_index: INTEGER) is
		require
			plane_index = -1 or plane_index = 0 or plane_index = 1
		local
			move: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			create plane.make(plane_orientation, create {IDEAL_CUBE_INDEX}.make(plane_index))
			create move.make(plane, create {ROTATE_COUNT}.make(1))
			rotate(move, User_rotate90_ms)
		end
	
feature {NONE} -- Cube IO
	
	save_cube is
		local
			f: RAW_FILE
		do
			if file_select /= Void then
				file_select.select_save_file
				if file_select.selected then
					create f.make_open_write(file_select.file_name)
					cube.ideal_cube.basic_store(f)
					f.close
				end
			end
		end
	
	load_cube is
		local
			loaded_cube: IDEAL_CUBE
			store: STORABLE
		do
			if file_select /= Void then
				file_select.select_open_file
				if file_select.selected then
					create store
					loaded_cube ?= store.retrieve_by_name(file_select.file_name)
					if loaded_cube /= Void then
						cube.set_ideal_cube(loaded_cube)
						notify_observer
					end
				end
			end
		end
	
feature {NONE} -- Demo loop
	
	process_cube is
		local
			exception_occured: BOOLEAN
			exception_description: STRING
		do
			if not exception_occured then
				demo_stopped := False
				from until stopping.is_true loop
					celebrate
					if not stopping then
						feature {SYSTEM_THREAD}.sleep_integer(3000)
						shuffle_cube
						if not stopping then
							feature {SYSTEM_THREAD}.sleep_integer(2000)
							solve_cube
						end
					end
				end
			else
			    if error_reporter /= Void then
					error_reporter.report(exception_description, "[Rubik's Cube] Demo loop terminated")
				end
			end
			demo_stopped := True
		ensure
			demo_stopped.is_true
		rescue
			exception_occured := True
			exception_description := "[" + last_exception_type + "] " + tag_name
			retry
		end
	
	celebrate is
		local
			flyby: FLYBY
			timer: HIGH_RESOLUTION_TIMER
			current_time: INTEGER
		do
			celebrating := True
			create flyby.make(get_spectator.angle, Time_to_celebrate_ms, 2, 1)
			create timer
			from until timer.elapsed_ms >= Time_to_celebrate_ms or stopping.is_true loop
				current_time := timer.elapsed_ms
				if current_time < Time_to_celebrate_ms then
					update_spectator_angle(flyby.now(current_time))
				end
			end
			if not stopping then
				update_spectator_angle(flyby.final)
			end
			celebrating := False
		end
	
	shuffle_cube is
		local 
			i: INTEGER
			random: RANDOM
			plane_index, plane_orientation, rotate_count: INTEGER
			plane: CUBE_PLANE_PARAMETERS
			orientations: ARRAY[INTEGER]
		do
			orientations := <<feature {ORIENTATION}.Horizontal, feature {ORIENTATION}.Vertical_perpendicular_to_x, feature {ORIENTATION}.Vertical_perpendicular_to_y>>
			create random.set_seed(feature {ENVIRONMENT}.tick_count)
			from i := 1 until i > shuffle_count or stopping.is_true loop
				plane_orientation := random_int(3, random) + 1 ; random.forth
				plane_index := random_int(3, random) - 1 ; random.forth
				rotate_count := random_int(3, random) + 1 ; random.forth
				if rotate_count = 3 then
					rotate_count := -1
				end
				create plane.make(orientations.item(plane_orientation), create {IDEAL_CUBE_INDEX}.make(plane_index))
				rotate(create {PLAYER_MOVE}.make(plane, create {ROTATE_COUNT}.make(rotate_count)), Shuffling_rotate90_ms)
				i := i + 1
			end
		end
	
	solve_cube is
		local
			solution: ARRAYED_LIST[PLAYER_MOVE]
		do
			solution := find_solution(cube.ideal_cube)
			from solution.start until solution.off or stopping.is_true loop
				rotate(solution.item, Solving_rotate90_ms)
				solution.forth
			end
		end
	
	rotate(move: PLAYER_MOVE rotate90_time_ms: INTEGER) is
		require
			move /= Void
			rotate90_time_ms > 0
		local
			total_time: INTEGER
			angle: ANGLE
			timer: HIGH_RESOLUTION_TIMER
			rotation_angle: DOUBLE
		do
			create timer
			total_time := rotate90_time_ms * move.rotate_count.count.abs
			from until timer.elapsed_ms >= total_time loop
				rotation_angle := timer.elapsed_ms / rotate90_time_ms * 90
				if rotation_angle > 0 and rotation_angle < 360 then
					if move.rotate_count.count > 0 then
						create angle.make(rotation_angle)
					else
						create angle.make(360 - rotation_angle)
					end
					cube.rotate_plane(move.plane, angle)
					notify_observer
				end
			end
			cube.rotate_ideal_cube(move)
			notify_observer
		end
	
feature {NONE} valid_target(t: VECTOR_3): BOOLEAN is
		do
			--Result := ((create {VECTOR_3}.make) - t).length < 1.0e-10 
			Result := t.x.abs < 1.0e-10 and t.y.abs < 1.0e-10 and t.z.abs < 1.0e-10 
		end
	
feature {NONE} Spectator_hor_angle_change_delta: ANGLE3D is
		once
			create Result.make(create {ANGLE}.make(5), create {ANGLE}.make(0))
		end
	
feature {NONE} Spectator_vert_angle_change_delta: ANGLE3D is
		once
			create Result.make(create {ANGLE}.make(0), create {ANGLE}.make(5))
		end
	
	
feature {NONE}
	cube: CUBE
	shuffle_count: INTEGER
	stopping, celebrating, demo_stopped: THREAD_SAFE_BOOLEAN
	Solving_rotate90_ms: INTEGER is 1000
	Shuffling_rotate90_ms: INTEGER is 300
	User_rotate90_ms: INTEGER is 1000
	Time_to_celebrate_ms: INTEGER is 10_000
	Spectator_distance_change_rate: DOUBLE is 1.05
	
invariant
	stopping /= Void
	celebrating /= Void
	demo_stopped /= Void
	cube /= Void
	shuffle_count >= 0
	valid_spectator(get_spectator)
	valid_target(target)
	
end
