
class
	FLYBY
	
inherit
	MATH_MISC	
	
create
	make
	
feature make(new_start_angle: like start_angle
			 new_total_time, new_horizontal_rotation_count, new_vertical_oscillation_count: INTEGER) is
		require
			new_start_angle /= Void and then valid_vert_angle(new_start_angle.vert_angle)
			new_total_time > 0
			new_horizontal_rotation_count >= 1
			new_vertical_oscillation_count >= 1
		do
			start_angle := new_start_angle
			total_time := new_total_time
			vertical_oscillation_count := new_vertical_oscillation_count
			horizontal_rotation_count := new_horizontal_rotation_count
		end
	
feature valid_vert_angle(angle: ANGLE): BOOLEAN is
		require
			angle /= Void
		do
			Result := angle.value < 90 or angle.value > 270
		end		
	
feature now(current_time: like total_time): like start_angle is	
		require
			valid_time(current_time)
		do
			create Result.make(horizontal_now(current_time), vertical_now(current_time))
		ensure
			Result /= Void	
		end
	
feature valid_time(time: like total_time): BOOLEAN is
		do
			Result := time >= 0 and time <= total_time
		end		
	
feature final: like start_angle is
		do
			Result := start_angle
		ensure
			Result /= Void	
		end		
	
feature {NONE} horizontal_now(current_time: like total_time): ANGLE is
		require
			valid_time(current_time)
		local
			add_angle: INTEGER
		do
			add_angle := current_time * horizontal_rotation_count * Full_horizontal_rotation // total_time 
			Result := start_angle.hor_angle + create {ANGLE}.make(add_angle \\ Max_angle_allowed)
		ensure
			Result /= Void
		end	
	
feature {NONE} vertical_now(current_time: like total_time): ANGLE is
		require
			valid_time(current_time)
		local
			original_up, moving_down: BOOLEAN
			delta, max_angle_from_horizontal_plane, angle_to_move: DOUBLE
			time_to_oscillation, current_time_in_oscillation, half_time_to_oscillation: DOUBLE
			progress: DOUBLE
		do
			if start_angle.vert_angle.value < 1.0e-10 then
				create Result.make(0)
			else	
				original_up := start_angle.vert_angle.value < 90
				max_angle_from_horizontal_plane := angle_from_horizontal_plane(start_angle.vert_angle)
				
				time_to_oscillation := total_time / vertical_oscillation_count
				current_time_in_oscillation := double_remainder(current_time, time_to_oscillation)
				
				half_time_to_oscillation := time_to_oscillation / 2
				if current_time_in_oscillation < half_time_to_oscillation then
					moving_down := original_up
				else
					moving_down := not original_up
				end
				
				progress := double_remainder(current_time_in_oscillation, half_time_to_oscillation) / half_time_to_oscillation 
					check progress >= 0 and progress <= 1 end
				angle_to_move := progress * (2 * max_angle_from_horizontal_plane) 
					check angle_to_move >= 0 and angle_to_move < 180 end
				if moving_down then
					create Result.make(max_angle_from_horizontal_plane)
					Result.sub(create {ANGLE}.make(angle_to_move))
				else
					create Result.make(Max_angle_allowed - max_angle_from_horizontal_plane)
					Result.add(create {ANGLE}.make(angle_to_move))
				end	
			end
		ensure
			Result /= Void and then angle_from_horizontal_plane(Result) <= angle_from_horizontal_plane(start_angle.vert_angle)
		end		
	
feature {NONE} angle_from_horizontal_plane(vert_angle: ANGLE): DOUBLE is
		require
			vert_angle /= Void and then valid_vert_angle(vert_angle)
		do
			if vert_angle.value < 90 then
				Result := vert_angle.value
			else
				Result := Max_angle_allowed - vert_angle.value
			end
		ensure
			Result >= 0 and Result < 90
		end		
	
feature {NONE}
	start_angle: ANGLE3D 
	total_time: INTEGER 
	horizontal_rotation_count: INTEGER 
	vertical_oscillation_count: INTEGER
	Full_horizontal_rotation: INTEGER is 360
	Max_angle_allowed: INTEGER is 360
	
invariant
	start_angle /= Void and then valid_vert_angle(start_angle.vert_angle)		
	total_time > 0
	horizontal_rotation_count >= 1
	vertical_oscillation_count >= 1
	
end 
