indexing
	description: "Resolution up to 1 ms"
	
class
    HIGH_RESOLUTION_TIMER
	
inherit
	EXCEPTIONS
		redefine
			default_create
		end
	
feature {NONE} default_create is
		local
			status: INTEGER
			frequency_ref: INTEGER_64
		do
			status := query_performance_frequency_win32_api($frequency_ref)
			if status = 0 or frequency_ref = 0 then
				raise("QueryPerformanceFrequency is not available.")
			end
			frequency := frequency_ref
			start := current_time_ms
		end
	
feature elapsed_ms: INTEGER is
		local 
			elapsed: INTEGER_64
		do
			elapsed := current_time_ms - start
			if elapsed <= feature {INTEGER}.max_value then 
				Result := elapsed.to_integer
			else
				raise("Timer overflow.")
			end	
		ensure
			Result >= 0
		end
	
feature {NONE} current_time_ms: INTEGER_64 is
		local
			counter: INTEGER_64
			status: INTEGER
		do
			status := query_performance_counter_win32_api($counter)
			if status = 0 or counter = 0 then
				raise("QueryPerformanceCounter is not available.")
			end
            Result := counter * 1000 // frequency
        ensure
			Result >= 0    
		end	
	
	
	query_performance_counter_win32_api(x: TYPED_POINTER[INTEGER_64]): INTEGER is
        external
			"C [macro <Windows.h>] (LARGE_INTEGER*): EIF_BOOLEAN"
        alias
			"QueryPerformanceCounter"
        end
	
	query_performance_frequency_win32_api(x: TYPED_POINTER[INTEGER_64]): INTEGER is
        external
			"C [macro <Windows.h>] (LARGE_INTEGER*): EIF_BOOLEAN"
        alias
			"QueryPerformanceFrequency"
        end
	
feature {NONE} start, frequency: INTEGER_64
	
invariant
	frequency > 0
	start >= 0
	
end 
