indexing
	notes: "(cube) plane - 9 bricks, (cube) side - 9 colors, cell - 1 color"
	
class
	IDEAL_CUBE
	
inherit
	STORABLE 
		redefine 
			default_create 
		end
	
feature bricks: ARRAYED_LIST[IDEAL_BRICK]
	
feature Total_bricks: INTEGER is 27
	
feature default_create is
		local 
			i: INTEGER
			p: IDEAL_BRICK_POSITION
		do
			create bricks.make(Total_bricks)
			from i := 1 until i > All_positions.count loop
				p := All_positions.i_th(i)
				bricks.extend(create {IDEAL_BRICK}.make(pristine_position_colors(p), p))
				i := i + 1
			end			
		ensure then
			is_solved	
		end
	
feature select_all_except_plane(plane: CUBE_PLANE_PARAMETERS): ARRAYED_LIST[IDEAL_BRICK] is	
		require
			plane /= Void
		local 
			i: INTEGER
		do
			create Result.make(Total_bricks - Bricks_on_plane)
			from i := 1 until i > bricks.count loop
				if not plane.has(bricks.i_th(i))  then
					Result.extend(bricks.i_th(i))
				end	
				i := i + 1
			end
		ensure
			Result /= Void 
			Result.count = Total_bricks - Bricks_on_plane
			not Result.has(Void)
		end
	
feature select_plane(plane: CUBE_PLANE_PARAMETERS): ARRAYED_LIST[IDEAL_BRICK] is
		require
			plane /= Void
		do
			create Result.make(Bricks_on_plane)
			bricks.do_if(agent Result.extend, agent plane.has)
		ensure
			Result /= Void 
			Result.count = Bricks_on_plane
			not Result.has(Void)
		end
	
feature play_move(move: PLAYER_MOVE) is
		require
			move /= Void
		do
			select_plane(move.plane).do_all(agent {IDEAL_BRICK}.rotate(move.plane.orientation, move.rotate_count))
		end
	
feature select_brick(p: IDEAL_BRICK_POSITION): IDEAL_BRICK is
		require
			p /= Void
		do
			from bricks.start until bricks.off loop
				if brick_has_same_position(bricks.item, p) then
					Result := bricks.item
				end
				bricks.forth
			end		
		ensure
			Result /= Void	
		end
	
feature is_solved: BOOLEAN is
		local 
			each_side_has_only_one_color, sides_colors_are_different_and_valid: BOOLEAN
		do
			each_side_has_only_one_color := all_sides_colors.for_all(agent side_has_only_one_color)
			sides_colors_are_different_and_valid := get_sides_colors.for_all(agent Side_colors.has)
			Result := each_side_has_only_one_color and sides_colors_are_different_and_valid
		end	
	
feature {NONE} pristine_position_colors(p: IDEAL_BRICK_POSITION): SIDE_COLORS is
		require
			p /= Void
		local
			front, back, left, right, top, bottom: INTEGER
		do
			if p.y.value = -1 then
				right := feature {COLOR}.Blue
			else
				right := feature {COLOR}.Black
			end
			
			if p.y.value = 1 then
				left := feature {COLOR}.Red
			else
				left := feature {COLOR}.Black
			end
			
			if p.x.value = 1 then
				front := feature {COLOR}.Green
			else
				front := feature {COLOR}.Black
			end
			
			if p.x.value = -1 then
				back := feature {COLOR}.Pink
			else
				back := feature {COLOR}.Black
			end
			
			if p.z.value = 1 then
				top := feature {COLOR}.Yellow
			else
				top := feature {COLOR}.Black
			end
			
			if p.z.value = -1 then
				bottom := feature {COLOR}.White
			else
				bottom := feature {COLOR}.Black
			end
			
			create Result.make(front, back, left, right, top, bottom)
		ensure
			Result /= Void
		end
	
	
feature {NONE} brick_has_same_position(brick: IDEAL_BRICK p: IDEAL_BRICK_POSITION): BOOLEAN is
		require
			brick /= Void
			p /= Void
		do
			Result := brick.position.is_equal(p)
		end
	
feature {NONE} side_has_only_one_color(colors: ARRAYED_LIST[INTEGER]): BOOLEAN is	
		require
			colors /= Void and then colors.count = Bricks_on_side
		do
			Result := colors.sequential_occurrences(colors.i_th(1)) = Bricks_on_side
		end
	
feature {NONE} get_sides_colors: ARRAYED_LIST[INTEGER] is
		local
			all_planes: like all_sides_colors
		do
			all_planes := all_sides_colors
			create Result.make(Cube_sides)
			from all_planes.start until all_planes.off loop
				Result.extend(all_planes.item.i_th(1))
				all_planes.forth
			end	
		ensure
			Result /= Void and then Result.count = Cube_sides
		end	
	
feature {NONE} one_side_colors(plane: ARRAYED_LIST[IDEAL_BRICK] side: DIRECTION): ARRAYED_LIST[INTEGER] is	
		require
			plane /= Void 
			plane.count = Bricks_on_plane
			not plane.has(Void)
			side /= Void
		do
			create Result.make(Bricks_on_side)
			from plane.start until plane.off loop
				Result.extend(plane.item.colors.select_color(side))
				plane.forth
			end
		ensure
			Result /= Void and then	Result.count = Bricks_on_side	
		end
	
feature {NONE} all_sides_colors: ARRAYED_LIST[ARRAYED_LIST[INTEGER]] is
		local 	
			i: INTEGER
			plane_parameters: CUBE_PLANE_PARAMETERS
			direction_for_plane: DIRECTION
		do
			create Result.make(Cube_sides)
			from i := 1 until i > Cube_sides loop
				plane_parameters ?= Plane_to_side_mapping.i_th(i).item(1)
				direction_for_plane ?= Plane_to_side_mapping.i_th(i).item(2)
				Result.extend(one_side_colors(select_plane(plane_parameters), direction_for_plane))
				i := i + 1
			end
		ensure
			Result /= Void 
			Result.count = Cube_sides 
			not Result.has(Void)
		end
	
feature {NONE} all_positions_set_with_bricks: BOOLEAN is
		local 
			i: INTEGER
			p: IDEAL_BRICK_POSITION
			not_found: BOOLEAN
		do
			from i := 1 until i > All_positions.count loop
				p := All_positions.i_th(i)
				if not bricks.there_exists(agent brick_has_same_position(?, p)) then
					not_found := True
				end
				i := i + 1
			end			
			Result := not not_found
		end
	
feature {NONE} all_colors_in_place: BOOLEAN is
		local 
			i: INTEGER
			all_cells_colors: ARRAYED_LIST[INTEGER]
			color_missed: BOOLEAN
		do
			create all_cells_colors.make(Cube_sides * Bricks_on_side)
			all_cells_colors.start
			all_sides_colors.do_all(agent all_cells_colors.merge_left)
			
			from i := 1 until i > Side_colors.count loop
				if all_cells_colors.occurrences(Side_colors.i_th(i)) < Bricks_on_side then
					color_missed := True
				end
				i := i + 1
			end
			
			Result := not color_missed
		end
	
feature {NONE} Plane_to_side_mapping: ARRAYED_LIST[ TUPLE[CUBE_PLANE_PARAMETERS, DIRECTION] ] is
		local
			plane: CUBE_PLANE_PARAMETERS
		once
			create Result.make(Cube_sides)
			
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_x, create {IDEAL_CUBE_INDEX}.make(1))
			Result.extend([plane, Direction.Front])
			
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_x, create {IDEAL_CUBE_INDEX}.make(-1))
			Result.extend([plane, Direction.Back])
			
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_y, create {IDEAL_CUBE_INDEX}.make(1))
			Result.extend([plane, Direction.Left])
			
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_y, create {IDEAL_CUBE_INDEX}.make(-1))
			Result.extend([plane, Direction.Right])
			
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(1))
			Result.extend([plane, Direction.Top])
			
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			Result.extend([plane, Direction.Bottom])
		ensure
			Result /= Void and then	Result.count = Cube_sides
		end
	
feature	{NONE} Direction: DIRECTION is
		once
			create Result
		end           
	
feature {NONE} Side_colors: ARRAYED_LIST[INTEGER] is	
		once
			create Result.make_from_array(<<feature {COLOR}.White, feature {COLOR}.Blue, feature {COLOR}.Green, 
											feature {COLOR}.Pink, feature {COLOR}.Red, feature {COLOR}.Yellow>>)
		ensure
			Result /= Void and then Result.count = Cube_sides
		end
	
feature {NONE} All_positions: ARRAYED_LIST[IDEAL_BRICK_POSITION] is	
		local
			x, y, z: INTEGER
			p: IDEAL_BRICK_POSITION
		once
			create Result.make(Total_bricks)
			from x := -1 until x > 1 loop
				from y := -1 until y > 1 loop
					from z := -1 until z > 1 loop
						create p.make(create {IDEAL_CUBE_INDEX}.make(x), create {IDEAL_CUBE_INDEX}.make(y), create {IDEAL_CUBE_INDEX}.make(z))
						Result.extend(p)
						z := z + 1
					end
					y := y + 1
				end
				x := x + 1
			end
		ensure
			Result /= Void and then Result.count = Total_bricks
		end
	
feature {NONE}
	Bricks_on_side, Bricks_on_plane: INTEGER is 9
	Cube_sides: INTEGER is 6
	
invariant
	bricks /= Void 
	bricks.count = Total_bricks
	not bricks.has(Void)
	all_positions_set_with_bricks
	all_colors_in_place
	
end	
