
class
	MISC
	
inherit
	EXCEPTIONS	
	
feature point_to_string(point: VECTOR_3): STRING is
		do
			Result := "[x = " + point.x.out + " y = " + point.y.out + " z = " + point.z.out + "]"
		ensure
			Result /= Void
		end
	
feature ideal_point_to_string(p: IDEAL_BRICK_POSITION): STRING is
		require
			p /= Void
		do
			Result := "[x = " + p.x.value.out + " y = " + p.y.value.out + " z = " + p.z.value.out + "]"
		ensure
			Result /= Void
		end
	
feature move_to_string(move: PLAYER_MOVE): STRING is
		require
			move /= Void
		do
			Result := "[Plane: "
			inspect 
				move.plane.orientation
			when feature {ORIENTATION}.Horizontal then
				Result.append_string("hor ")
			when feature {ORIENTATION}.Vertical_perpendicular_to_x then
				Result.append_string("vert_x_perp ")
			when feature {ORIENTATION}.Vertical_perpendicular_to_y then
				Result.append_string("vert_y_perp ")
			end	
			Result.append_integer(move.plane.index.value)
			Result.append_string(" Rotate: " + move.rotate_count.count.out + "]")
		ensure
			Result /= Void
		end
	
feature load_resource_as_stream(resource_name, file_name: STRING): SYSTEM_STREAM is
		require
			resource_name /= Void
			file_name /= Void
		local	
			resource_assembly: ASSEMBLY
			resource_assembly_name: ASSEMBLY_NAME
		do
			create resource_assembly_name.make
			resource_assembly_name.set_name(file_name)
			resource_assembly := feature {ASSEMBLY}.load_assembly_name(resource_assembly_name)
			Result := resource_assembly.get_manifest_resource_stream(resource_name)
			if Result = Void then
				raise("Can't load resource " + resource_name + " from " + file_name)
			end
		ensure
			Result /= Void	
		end
	
feature show_error_message(text, caption: STRING) is
		require
			text /= Void
			caption /= Void 
		local 
			dummy: WINFORMS_DIALOG_RESULT
		do
			dummy := feature {WINFORMS_MESSAGE_BOX}.show(text, caption,
														 feature {WINFORMS_MESSAGE_BOX_BUTTONS}.ok,
														 feature {WINFORMS_MESSAGE_BOX_ICON}.error)
		end
	
feature load_boolean_from_config(key: STRING): BOOLEAN is	
		require
			key /= Void
		do
			Result := load_value_from_config(key).is_equal("True")
		end
	
feature load_non_negative_integer_from_config(key: STRING): INTEGER is
		require
			key /= Void
		local		
			key_value: STRING
		do
			key_value := load_value_from_config(key)
			if key_value.is_integer then
				Result := key_value.to_integer
				if Result < 0 then
					Result := -1
				end
			else
				Result := -1	
			end	
		ensure
			Result >= 0 or Result = -1			
		end
	
feature last_exception_type: STRING is
		local
			e: EXCEPTION
		do
			e := feature {EXCEPTION_MANAGER}.last_exception
			if e /= Void then
				Result := e.get_type.full_name
			else
				Result := ""	
			end
		ensure
			Result /= Void
		end	
	
feature output_debug_string(s: STRING) is
		require
			s /= Void
		do
			feature {SYSTEM_DLL_TRACE}.write_line_string(s)
		end
	
feature {NONE} load_value_from_config(key: STRING): STRING is		
		require
			key /= Void
		local		
			settings: SYSTEM_DLL_NAME_VALUE_COLLECTION
		do
			settings := feature {SYSTEM_DLL_CONFIGURATION_SETTINGS}.app_settings
			Result := settings.get_string(key)
		ensure
			Result /= Void
		end
	
	
end 
