indexing
	attribute: create {TEST_FIXTURE_ATTRIBUTE}.make	end
	
class
	TEST_CUBE_SOLVER
	
inherit
	TEST_ROOT
	CUBE_SOLVER
	
feature test_parse_one_entry is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make	end
		local 
			actual: ARRAYED_LIST[STRING]
		do
			actual := parse_moves_string("[+ rf]")
			feature {ASSERT}.are_equal_integer_integer(1, actual.count)
			assert_are_equal_string_string("+rf", actual.i_th(1))
		end
	
feature test_parse_two_entries is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			actual: ARRAYED_LIST[STRING]
		do
			actual := parse_moves_string("[+ u, - r]")
			feature {ASSERT}.are_equal_integer_integer(2, actual.count)
			assert_are_equal_string_string("+u", actual.i_th(1))
			assert_are_equal_string_string("-r", actual.i_th(2))
		end
	
feature test_long_string_to_moves is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			actual: ARRAYED_LIST[PLAYER_MOVE]
			expected: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			actual := long_string_to_moves("-rf")
			feature {ASSERT}.are_equal_integer_integer(3, actual.count)
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_x, create {IDEAL_CUBE_INDEX}.make(-1)) 			
			create expected.make(plane, create {ROTATE_COUNT}.make(-1))			
			feature {ASSERT}.is_true(expected.is_equal(actual.i_th(1)) or 
									 expected.is_equal(actual.i_th(2)) or 
									 expected.is_equal(actual.i_th(3)))
		end
	
	
feature test_short_string_to_move_1 is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			actual, expected: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			actual := short_string_to_move("-u")
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(1))
			create expected.make(plane, create {ROTATE_COUNT}.make(-1))			
			assert_are_equal_move_move(expected, actual)
		end
	
feature test_short_string_to_move_2 is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			actual, expected: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			actual := short_string_to_move("+r")
			create plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_y, create {IDEAL_CUBE_INDEX}.make(-1))
			create expected.make(plane, create {ROTATE_COUNT}.make(-1))			
			assert_are_equal_move_move(expected, actual)
		end
	
feature test_solve_unscrambled is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			cube: IDEAL_CUBE
		do
			create cube
			feature {ASSERT}.are_equal(0, find_solution(cube).count)
		end
	
feature test_solve_simple is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			cube: IDEAL_CUBE
			move: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			create cube
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			cube.play_move(move)
			find_solution(cube).do_all(agent cube.play_move)
			feature {ASSERT}.is_true(cube.is_solved)
		end
	
feature test_solve_moved_centers is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			cube: IDEAL_CUBE
			move: PLAYER_MOVE
			plane: CUBE_PLANE_PARAMETERS
		do
			create cube
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(0))
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			cube.play_move(move)
			find_solution(cube).do_all(agent cube.play_move)
			feature {ASSERT}.is_true(cube.is_solved)
		end
	
feature test_number_of_moves_to_solve is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			cube: IDEAL_CUBE
			move: PLAYER_MOVE
			solution: ARRAYED_LIST[PLAYER_MOVE]
			plane: CUBE_PLANE_PARAMETERS
		do
			create cube
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1)) 
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			cube.play_move(move)
			solution := find_solution(cube)
			feature {ASSERT}.are_equal(128, solution.count)	
		end
	
feature test_remove_redundant_moves_two_consecutive_opposites is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move, opposite_move: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			original.extend(move)
			
			create opposite_move.make(plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(opposite_move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(0, reduced.count)	
		end
	
feature test_remove_redundant_moves_not_consecutive_opposites is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move, opposite_move, intermidiate_with_same_orientation: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane, intermidiate_plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			create intermidiate_plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(0))
			
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			original.extend(move)
			
			create intermidiate_with_same_orientation.make(intermidiate_plane, create {ROTATE_COUNT}.make(1))								
			original.extend(intermidiate_with_same_orientation)
			
			create opposite_move.make(plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(opposite_move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(1, reduced.count)	
		end
	
	
feature test_remove_redundant_moves_opposite_pair_inside_different_opposite_pair is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move, opposite_move: PLAYER_MOVE
			intermidiate_with_different_orientation, opposite_intermidiate_with_different_orientation: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane, intermidiate_plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			create intermidiate_plane.make(feature {ORIENTATION}.Vertical_perpendicular_to_x, create {IDEAL_CUBE_INDEX}.make(0))
			
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			original.extend(move)
			
			create intermidiate_with_different_orientation.make(intermidiate_plane,	create {ROTATE_COUNT}.make(1))								
			original.extend(intermidiate_with_different_orientation)
			
			create opposite_intermidiate_with_different_orientation.make(intermidiate_plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(opposite_intermidiate_with_different_orientation)
			
			create opposite_move.make(plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(opposite_move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(0, reduced.count)	
		end
	
feature test_remove_redundant_moves_two_same_moves is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move, sum_move: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			
			create move.make(plane, create {ROTATE_COUNT}.make(1))								
			original.extend(move)
			original.extend(move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(1, reduced.count)	
			
			create sum_move.make(plane, create {ROTATE_COUNT}.make(2))								
			assert_are_equal_move_move(sum_move, reduced.i_th(1))
		end
	
feature test_remove_redundant_moves_two_same_moves_producing_overflow is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move, sum_move: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			
			create move.make(plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(move)
			original.extend(move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(1, reduced.count)	
			
			create sum_move.make(plane, create {ROTATE_COUNT}.make(2))								
			assert_are_equal_move_move(sum_move, reduced.i_th(1))
		end
	
feature test_remove_redundant_moves_four_same_moves is
		indexing
			attribute: create {TEST_ATTRIBUTE}.make end
		local 
			move: PLAYER_MOVE
			original, reduced: ARRAYED_LIST[PLAYER_MOVE]
			plane: CUBE_PLANE_PARAMETERS
		do
			create original.make(0)
			create plane.make(feature {ORIENTATION}.Horizontal, create {IDEAL_CUBE_INDEX}.make(-1))
			
			create move.make(plane, create {ROTATE_COUNT}.make(-1))								
			original.extend(move)
			original.extend(move)
			original.extend(move)
			original.extend(move)
			
			reduced := remove_redundant_moves(original)
			feature {ASSERT}.are_equal(0, reduced.count)	
		end
	
end
