
class
	VIEW
	
inherit
	OBSERVER
	D3D_APP
		rename
			make as make_d3d
		end
	MATH_CONST
	MISC
	
create
	make
	
feature {NONE} make(new_model: MODEL new_window_handle: POINTER new_form: WINFORMS_FORM 
					start_in_fullscreen, new_software_rendering, black_background: BOOLEAN 
					start_size: DRAWING_SIZE) is
		require
			new_model /= Void
			new_window_handle /= Void
			new_form /= Void
		local
			background_color: DRAWING_COLOR
		do
			model := new_model			
			create timer
			create fps_buffer.make(100)
			initialize_counters			
			
			if black_background then
				background_color := feature {DRAWING_COLOR}.black	
			else
				background_color := feature {DRAWING_COLOR}.dark_gray
			end
			make_d3d(new_window_handle, new_form, start_in_fullscreen, new_software_rendering, background_color, start_size)
		end
	
	
feature {NONE} update_counters is
		local 
			now, elapsed: INTEGER
		do
			now := timer.elapsed_ms
			if previous_update_time /= 0 then
				elapsed := now - previous_update_time
				if elapsed > 0 and elapsed <= 100 then
					if pc_min /= Void and pc_max /= Void and pc_avg /= Void then				
						fps_buffer.extend(1000 // elapsed)
						pc_min.set_raw_value(fps_buffer.min)
						pc_avg.set_raw_value(fps_buffer.avg)
						pc_max.set_raw_value(fps_buffer.max)
					end	
				end	
			end
			previous_update_time := now		
		end	
	
feature {NONE} custom_initialize_graphics is 
		do
			create brick_painter.make(render_device)					
		end
	
feature {NONE} custom_render is 
		do
			setup_matrices
			if model.scene /= Void then
				model.scene.render(brick_painter)
			end	
			update_counters
		end
	
feature {NONE} setup_matrices is
		local
			axes: VECTOR_3
			aspect_ratio: REAL -- to compensate non square window size
		do
			axes.make(0.0, 0.0, 1.0) 
			render_device.Transform.set_view(feature {MATRIX}.look_at_lh(model.get_spectator.position, model.target, axes))		
			if fullscreen then
				aspect_ratio := form.width / form.height
			else
				aspect_ratio := form.client_size.width / form.client_size.height 	
			end	
			render_device.Transform.set_projection(feature {MATRIX}.perspective_fov_lh(Pi / 4.0, aspect_ratio, 1.0, 100))
		end
	
feature {NONE} initialize_counters is
		local 
			counters_not_installed: BOOLEAN
		do
			if not counters_not_installed then
				create pc_min.make("Rubik's Cube", "Min FPS", False)
				create pc_avg.make("Rubik's Cube", "Avg FPS", False)
				create pc_max.make("Rubik's Cube", "Max FPS", False)
			end	
		rescue
			counters_not_installed := True
			retry	
		end	
	
	
feature {NONE}
	model: MODEL
	brick_painter: BRICK_PAINTER
	previous_update_time: INTEGER
	pc_min, pc_max, pc_avg: SYSTEM_DLL_PERFORMANCE_COUNTER
	fps_buffer: STATISTIC_BOUNDED_BUFFER
	timer: HIGH_RESOLUTION_TIMER
	
invariant
	model /= Void
	brick_painter /= Void
	fps_buffer /= Void
	timer /= Void
	
end 
